import os
import sys

module_path = os.path.dirname(os.path.realpath(__file__))
sys.path.append(module_path)

import torch
import torch.nn as nn
import albumentations as A
import albumentations.pytorch as AT
import gdown

from data import BrainMRIDataset
from models import create_model
from utils import dcm2image, study2series_dict

__all__ = ['PulseSequenceClassifier']

model_urls = {
    'resnet18': '1StQN8AVKb4jwWZNjHerPAg5Kw3hkHrKN',}


class PulseSequenceClassifier():
    def __init__(self, model_name="resnet18", device="cpu"):
        self.device = torch.device(device)

        self.model_name = model_name

        supported_models = list(model_urls.keys())
        if model_name not in supported_models:
            raise NotImplementedError('Currently supported models are', list(model_urls.keys()))

        self.model = create_model(self.model_name, pretrained=False, n_classes=4)
        self.model = self.model.to(self.device)
        self.model = nn.DataParallel(self.model)

        self.transform = A.Compose([
            A.Resize(256, 256),
            A.Normalize(),
            AT.ToTensor()
        ])

        self.label_dict = dict([
            (0, "FLAIR"),
            (1, "T1"),
            (2, "T2"),
            (3, "DWI"),
        ])

    def from_pretrained(self):
        checkpoint_path = module_path + "/{}.pt".format(self.model_name)
        if not os.path.exists(checkpoint_path):
            checkpoint_url = f'https://drive.google.com/uc?id={model_urls[self.model_name]}'
            gdown.cached_download(checkpoint_url, checkpoint_path)
        self.model.load_state_dict(torch.load(checkpoint_path, map_location=self.device))
        return self

    def predict_instance(self, instance_path):
        image = dcm2image(instance_path)
        image = self.transform(image=image)["image"]

        with torch.no_grad():
            self.model.eval()
            image = image.unsqueeze(0).to(self.device)

            output = self.model(image)
            pred = torch.max(output, 1)[1].detach().cpu().numpy()

        return self.label_dict[pred[0]]

    def predict_study(self, study_path):
        self.series_dict = study2series_dict(study_path)
        self.loader = torch.utils.data.DataLoader(
            BrainMRIDataset(
                self.series_dict,
                image_transform=self.transform
            ),
            batch_size=len(self.series_dict), shuffle=False
        )

        with torch.no_grad():
            self.model.eval()
            series_uids, images = next(iter(self.loader))
            images = images.to(self.device)

            outputs = self.model(images)
            preds = torch.max(outputs, 1)[1].detach().cpu().numpy()

        for i, series_uid in enumerate(series_uids):
            self.series_dict[series_uid] = self.label_dict[preds[i]]

        return self.series_dict




import torch.nn as nn
import torchvision

def create_model(model_name, pretrained, n_classes):
    if model_name == "resnet18":
        model = torchvision.models.resnet18(pretrained=pretrained)
        model.fc = nn.Linear(model.fc.in_features, n_classes)

    return model



import glob
import numpy as np
from pydicom import dcmread
from pydicom.pixel_data_handlers.util import apply_voi_lut

def isaxial(IOP):
    IOP = [round(float(s)) for s in IOP]
    if IOP == [1, 0, 0, 0, 1, 0]:
        return True
    else:
        return False

def dcm2image(dcm_path):
    dcm = dcmread(dcm_path)
    array = apply_voi_lut(dcm.pixel_array, dcm)
    if dcm.PhotometricInterpretation == "MONOCHROME1":
        array = np.amax(array) - array

    array = array - np.min(array)
    array = array / np.max(array)
    array = array * 255
    array = array.astype(np.uint8)

    image = np.stack([array] * 3, axis=0)
    image = np.transpose(image, (1, 2, 0))

    return image

def study2series_dict(study_path):
    series_dict = {}

    dcm_paths = glob.glob("{}/*.dcm".format(study_path))
    for dcm_path in dcm_paths:
        dcm = dcmread(dcm_path)
        series_uid = dcm.SeriesInstanceUID
        try:
            if series_uid not in series_dict:
                series_dict[series_uid] = [[dcm_path, int(dcm.InstanceNumber), isaxial(dcm.ImageOrientationPatient)]]
            else:
                series_dict[series_uid].append([dcm_path, int(dcm.InstanceNumber), isaxial(dcm.ImageOrientationPatient)])
        except:
            pass

    for series_uid in list(series_dict.keys()):
        series_instance_planes = [instance[2] for instance in series_dict[series_uid]]
        if (sum(series_instance_planes) / len(series_instance_planes)) < 0.8:
            del series_dict[series_uid]

    for series_uid in list(series_dict.keys()):
        s = sorted(series_dict[series_uid], key=lambda instance: instance[1])
        image = dcm2image(s[len(s) // 2][0])
        series_dict[series_uid] = image

    return series_dict


from torch.utils.data import Dataset

class BrainMRIDataset(Dataset):
    def __init__(self, series_dict, image_transform=None):
        self.series_dict = series_dict
        self.image_transform = image_transform

    def __len__(self):
        return len(self.series_dict)

    def __getitem__(self, idx):
        series_uids = list(self.series_dict.keys())

        image = self.series_dict[series_uids[idx]]
        if self.image_transform is not None:
            image = self.image_transform(image=image)["image"]

        return series_uids[idx], image



from pkg_resources import parse_requirements
from setuptools import setup

setup(
    name='brainmri_ps',
    version='0.1.6',
    packages=['brainmri_ps'],
    url='',
    license='LICENSE.txt',
    author='lhkhiem, tuantran',
    author_email='',
    description='Automatically classify Brain MRI series by pulse sequence types: DWI, FLAIR, T1, T2, and Others',
    install_reqs=parse_requirements('requirements.txt'),
)